﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;

namespace SVIctl_CSharp
{
    /// <summary>
    /// iniファイル操作クラス
    /// </summary>
    public static class IniFileController
    {
        #region ▼ Field
        /// <summary>
        /// kernel32.dllのファイルパス
        /// </summary>
        private const string kernelDllName = "kernel32.dll";
        /// <summary>
        /// 実行ファイルのアプリケーション名
        /// </summary>
        private static readonly string appName;
        /// <summary>
        /// iniファイルの保存場所
        /// </summary>
        private static string iniFilePath;
        /// <summary>
        /// iniファイルから文字列を取得します。
        /// </summary>
        /// <param name="lpAppName">セクション</param>
        /// <param name="lpKeyName">キー</param>
        /// <param name="lpDefault">取得できなかった場合の初期値</param>
        /// <param name="lpReturnedString">取得した文字列</param>
        /// <param name="nSize">lpReturnedStringのバッファサイズ</param>
        /// <param name="lpFileName">iniファイルの保存場所</param>
        /// <returns>取得した文字列の文字数</returns>
        [DllImport(IniFileController.kernelDllName)]
        private static extern uint GetPrivateProfileString(string lpAppName, string lpKeyName, string lpDefault, StringBuilder lpReturnedString, uint nSize, string lpFileName);
        /// <summary>
        /// iniファイルに文字列を書き込みます。 
        /// </summary>
        /// <param name="lpAppName">セクション</param>
        /// <param name="lpKeyName">キー</param>
        /// <param name="lpString">書き込む文字列</param>
        /// <param name="lpFileName">iniファイルの保存場所</param>
        /// <returns>処理結果。true..正常、false..失敗</returns>
        [DllImport(IniFileController.kernelDllName, SetLastError = true)]
        private static extern bool WritePrivateProfileString(string lpAppName, string lpKeyName, string lpString, string lpFileName);

        /// <summary>
        /// iniファイル読み書き用のキー
        /// </summary>
        public enum Keys
        {
            SlaveAddress,
            SubAddress,
            WriteValue,
            ReadCounts,
            WordAddressMode,
            SettingFileName,
        }
        #endregion

        #region ▼ Property
        #endregion

        #region ▼ Constructor
        /// <summary>
        /// コンストラクタ
        /// </summary>
        static IniFileController()
        {
            // アプリケーション名を取得
            IniFileController.appName = Assembly.GetEntryAssembly().GetName().Name;

            // iniファイルの保存場所を定義（実行ファイルと同じ場所に保存）
            IniFileController.iniFilePath = Path.GetDirectoryName(Assembly.GetEntryAssembly().Location);
            IniFileController.iniFilePath = Path.Combine(IniFileController.iniFilePath, IniFileController.appName + ".ini");
        }
        #endregion

        #region ▼ Event
        #endregion

        #region ▼ Private Method
        /// <summary>
        /// Keysを対応する文字列に変換します。
        /// </summary>
        /// <param name="key">変換対象のkeys列挙子</param>
        /// <returns>変換後の文字列</returns>
        private static string ConvertKeyNameEnumToString(IniFileController.Keys key)
        {
            // 戻り値
            string result = null;

            switch (key)
            {
                case Keys.SlaveAddress:
                    result = "SlaveAddress";
                    break;
                case Keys.SubAddress:
                    result = "SubAddress";
                    break;
                case Keys.WriteValue:
                    result = "Value";
                    break;
                case Keys.ReadCounts:
                    result = "ReadCounts";
                    break;
                case Keys.WordAddressMode:
                    result = "WordAddress";
                    break;
                case Keys.SettingFileName:
                    result = "SettingFileName";
                    break;
                default:
                    break;
            }

            return result;
        }
        #endregion

        #region ▼ Public Method
        /// <summary>
        /// iniファイルから文字列を取得します。
        /// </summary>
        /// <param name="key">キー</param>
        /// <returns>取得した文字列</returns>
        public　static string GetString(IniFileController.Keys key)
        {
            StringBuilder targetText = new StringBuilder(1024);
            IniFileController.GetPrivateProfileString(IniFileController.appName, IniFileController.ConvertKeyNameEnumToString(key), "", targetText, (uint)targetText.Capacity, IniFileController.iniFilePath);
            return targetText.ToString();
        }

        /// <summary>
        /// iniファイルに文字列を書き込みます。 
        /// </summary>
        /// <param name="key">キー</param>
        /// <param name="targetValue">書き込む文字列</param>
        /// <returns>処理結果。true..正常、false..失敗</returns>
        public static bool WriteString(IniFileController.Keys key, string targetValue)
        {
            return IniFileController.WritePrivateProfileString(IniFileController.appName, IniFileController.ConvertKeyNameEnumToString(key), targetValue, IniFileController.iniFilePath);
        }

        /// <summary>
        /// iniファイルからboolを取得します。
        /// </summary>
        /// <param name="key">キー</param>
        /// <returns>取得結果</returns>
        public static bool GetBool(IniFileController.Keys key)
        {
            return (IniFileController.GetString(key).ToLower() == "true");
        }


        /// <summary>
        /// iniファイルにboolを書き込みます。
        /// </summary>
        /// <param name="key">キー</param>
        /// <param name="targetValue">書き込む値</param>
        /// <returns>処理結果。true..正常、false..失敗</returns>
        public static bool WriteBool(IniFileController.Keys key, bool targetValue)
        {
            return IniFileController.WriteString(key, targetValue.ToString());
        }
        #endregion

        #region ▼ Override Method
        #endregion
    }
}
