﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;

namespace SVIctl_CSharp
{

    // SVIボード制御用クラス
    public static class SVIControl
    {
        // デバイスオープンフラグ
        public static bool deviceOpenFlag = false;


        /// <summary>
        /// 画像入力ライブラリ開始処理
        /// </summary>
        public static void ApiInit()
        {
            // 制御ライブラリ初期化
            // InitとEndはアプリ起動時と終了時に1回のみ実行すること
            NativeMethods_SVI.Init();
        }


        /// <summary>
        /// 画像入力ライブラリ終了処理
        /// </summary>
        public static void ApiEnd()
        {
            // 制御ライブラリ終了処理
            NativeMethods_SVI.End();
        }


        /// <summary>
        /// 接続ボード数取得
        /// </summary>
        /// <returns>接続ボード数</returns>
        public static UInt32 GetBoardNum()
        {
            // 接続ボード数取得
            IntPtr bufBoardNum = new IntPtr();
            bufBoardNum = Marshal.AllocHGlobal(4);

            UInt32 ret = NativeMethods_SVI.SVIUSB20_EnumDevice(NativeMethods_SVI.APP_WHO_CTL, bufBoardNum, System.IntPtr.Zero);

            if (ret == NativeMethods_SVI.RET_NORMAL)
            {
                return (UInt32)(Marshal.ReadInt32(bufBoardNum));
            }
            else
            {
                // 接続ボードなし
                return 0;
            }
        }


        /// <summary>
        /// 接続ボード情報取得
        /// </summary>
        /// <returns>接続ボード情報 [31:16] = 接続ボード番号, [15:0] = Windows管理番号</returns>
        public static UInt32[] GetBoardInfo()
        {
            // 接続ボード数
            UInt32 boardNum = 0;

            // 接続ボード数の格納ポインタ
            IntPtr bufBoardNum = new IntPtr();
            bufBoardNum = Marshal.AllocHGlobal(4);

            UInt32 ret = NativeMethods_SVI.SVIUSB20_EnumDevice(NativeMethods_SVI.APP_WHO_CTL, bufBoardNum, System.IntPtr.Zero);

            if (ret == NativeMethods_SVI.RET_NORMAL)
            {
                boardNum =  (UInt32)Marshal.ReadInt32(bufBoardNum);

                // 戻り値用配列
                UInt32[] boardInfoArray = new UInt32[boardNum];

                // 接続ボード情報格納ポインタ
                IntPtr bufBoardInfo = new IntPtr();
                int size = (int)(Marshal.SizeOf(typeof(UInt32)) * boardNum);
                bufBoardInfo = Marshal.AllocHGlobal(size);

                // 接続ボード情報取得
                NativeMethods_SVI.SVIUSB20_EnumDevice(NativeMethods_SVI.APP_WHO_CTL, bufBoardNum, bufBoardInfo);

                for (int i = 0; i < boardNum; i++)
                {
                    boardInfoArray[i] = (UInt32)(Marshal.ReadInt32(bufBoardInfo, i * 4));
                }

                Marshal.FreeCoTaskMem(bufBoardNum);
                Marshal.FreeCoTaskMem(bufBoardInfo);

                return boardInfoArray;
            }
            else
            {
                // 接続ボードなし
                NativeMethods_SVI.End();
                return null;
            }
        }


        /// <summary>
        /// SVIデバイスオープン処理
        /// </summary>
        /// <param name="boardInfo">複数のボードが接続された場合、GetBoardInfo()で取得した値を代入</param>
        public static void DeviceOpen(UInt32 boardInfo)
        {
            UInt32 ret;

            // 接続デバイス選択
            NativeMethods_SVI.DeviceSelect(boardInfo);

            // デバイスドライバをオープン
            ret = NativeMethods_SVI.Open(NativeMethods_SVI.APP_WHO_CTL);

            SVIControl.deviceOpenFlag = true;
        }


        /// <summary>
        /// 接続済デバイスのクローズ処理
        /// </summary>
        public static void DeviceClose()
        {
            UInt32 ret;

            // デバイスドライバクローズ
            ret = NativeMethods_SVI.Close(NativeMethods_SVI.APP_WHO_CTL);

            // 接続ボード開放
            NativeMethods_SVI.DeviceRelease();

            SVIControl.deviceOpenFlag = false;
        }


        /// <summary>
        /// I2C Write処理
        /// </summary>
        /// <param name="slaveAddress">スレーブアドレス</param>
        /// <param name="subAddress">サブアドレス(8bit or 16bit)</param>
        /// <param name="data">書き込みデータ</param>
        /// <param name="wordAccess">WordAddressモード設定</param>
        /// <param name="restartCondtion">RestartCondition設定</param>
        /// <returns>正常:0x1, エラー:null</returns>
        public static UInt32? SVI_I2C_OneBlockWrite(UInt32 slaveAddress, UInt32 subAddress, byte[] data, bool wordAccess = false, bool restartCondtion = false)
        {

            // 引数成形
            if (data == null)
            {
                return null;
            }

            byte[] dataBuf;
            if (wordAccess)
            {
                dataBuf = new byte[data.Length + 2];
                dataBuf[0] = (byte)(subAddress >> 8);
                dataBuf[1] = (byte)(subAddress & 0xFF);

                for (int i = 0; i < data.Length; i++)
                {
                    dataBuf[i + 2] = data[i];
                }
            }
            else
            {
                dataBuf = new byte[data.Length + 1];
                dataBuf[0] = (byte)(subAddress & 0xFF);

                for (int i = 0; i < data.Length; i++)
                {
                    dataBuf[i + 1] = data[i];
                }
            }

            int len = dataBuf.Length;

            // 送信データ格納用ポインタ
            IntPtr ptr = Marshal.AllocCoTaskMem(len);

            // マネージド配列(C#側)からアンマネージドメモリポインター(DLL側)にデータをコピー
            Marshal.Copy(dataBuf, 0, ptr, len);

            // I2C Block Write API実行
            UInt32 ret;
            if (restartCondtion)
            {
                ret = NativeMethods_SVI.SVIUSB20_I2COneBlockWrite(slaveAddress, (UInt32)len, 0x80000000, ptr);
            }
            else
            {
                ret = NativeMethods_SVI.SVIUSB20_I2COneBlockWrite(slaveAddress, (UInt32)len, 0x0, ptr);
            }
            
            // 送信データ格納用メモリ空間を開放
            Marshal.FreeCoTaskMem(ptr);

            // 正常に通信が行えた場合、NativeMethods.RET_NORMAL(0x1)を返す
            if (ret == NativeMethods_SVI.RET_NORMAL)
            {
                return ret;
            }
            else
            {
                return null;
            }
        }


        /// <summary>
        /// I2C Read処理
        /// </summary>
        /// <param name="slaveAddress">スレーブアドレス</param>
        /// <param name="subAddress">サブアドレス</param>
        /// <param name="dataLen">Readデータ長</param>
        /// <param name="wordAccess">WordAddressモード設定</param>
        /// <param name="restartCondition">RestartCondition設定</param>
        /// <returns>読み込みデータ(バイト配列)</returns>
        public static byte[] SVI_I2C_OneBlockRead(UInt32 slaveAddress, UInt32 subAddress, UInt32 dataLen, bool wordAccess = false, bool restartCondition = false)
        {
            // 読み込みバイト長(4byte未満の場合もAPIには4byteと指定)
            UInt32 len = dataLen;

            // 受信バッファの最小byte長は4byte
            byte[] receiveBuffer = new byte[len];

            // 戻り値格納用バッファの最小byte長は1byte (引数で指定されたdataLenと一致)
            byte[] ret_value = new byte[dataLen];

            // 受信データ格納用ポインタ
            IntPtr ptr = Marshal.AllocCoTaskMem((int)len);

            // Read時サブアドレス指定用ダミーデータ
            byte[] dummyData = { };

            UInt32 ret;
            if (restartCondition)
            {
                // サブアドレス指定
                SVI_I2C_OneBlockWrite(slaveAddress, subAddress, dummyData, wordAccess, restartCondition);

                // I2C Block Read API実行
                ret = NativeMethods_SVI.SVIUSB20_I2COneBlockRead(slaveAddress, len, 0x80000000, ptr);
            }
            else
            {
                // サブアドレス指定
                SVI_I2C_OneBlockWrite(slaveAddress, subAddress, dummyData, wordAccess, restartCondition);

                // I2C Block Read API実行
                ret = NativeMethods_SVI.SVIUSB20_I2COneBlockRead(slaveAddress, len, 0x0, ptr);
            }

            // アンマネージドメモリポインター(DLL側)からマネージド配列(C#側)にデータをコピー
            Marshal.Copy(ptr, receiveBuffer, 0, (int)len);

            // 受信データ格納用メモリ空間を開放
            Marshal.FreeCoTaskMem(ptr);

            // 正常に通信が行えた場合、読み取りデータをバイト配列で返す
            if (ret == NativeMethods_SVI.RET_NORMAL)
            {
                // 受信バッファから戻り値格納用バッファへdataLenのbyte長のデータをコピー
                Array.Copy(receiveBuffer, 0, ret_value, 0, dataLen);

                // Debug
                Console.Write("Data: ");
                foreach (byte d in ret_value)
                {
                    Console.Write(d.ToString("X2"));
                }
                Console.WriteLine("");

                return ret_value;
            }
            else
            {
                return null;
            }
        }
    }
}
